#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   abaqus2fistr.py
#
#       abaqus から fistr へメッシュ変換
#       node番号、element番号の定義は、昇順に定義されている事が必要。
#       これらが昇順でない場合は、変換できるが、極端に実行速度が遅くなる。
#
#       abaqusType  node    fistrType
#       ---------------------------------------------
#       C3D4        4       341         四面体1次（tetra）
#       C3D10       10      342         四面体2次
#       C3D8        8       361         六面体1次（hexa）
#       C3D20       20      362         六面体2次
#       C3D6        6       351         五面体1次（prism）
#       C3D15       15      352         五面体2次
#       STRI35      3       (S341)      三角形1次表面要素
#       S6          6       (S342)      三角形2次表面要素
#       S4R5        4       (S361)      四角形1次表面要素
#       S8          8       (S362)      四角形2次表面要素
#
#
#   15/03/19    新規作成
#      03/27    mashDataを数値で読み込みに変更
#      03/29    SGROUPを設定
#      03/30    5面体要素を追加
#      04/09    selValNameを修正
#      04/13    separateElset:高速化（直接global変数を操作する様に修正）
#      04/29    elementNoをインデックスでそのまま保存して検索を高速化
#      05/07    elementListデータをsortしてface面を検索（高速化の為）
#      05/08    nodeNoをインデクスでそのnodeを含むelement、typeのlist作成（高速化）
#      05/21    その他のsurGroupList「otherS」グループを作成
#      09/01    windows対応のため、os.sepに修正
#      09/02    "\n"→os.linesepに修正
#

import os
import sys

ls = os.linesep           #行末コードを定義

abaName = ""
nodeList = []           #NODE
elementList = []        #ELEMENT
surfaceList = []
surGroupList = []       #SGROUP
elsetList = []
ndsetList = []          #NGROUP
nElementType = 0        #elementTypeの種類
nElsetSur = []
nElsetEl = []
elNodeList = []         #element(表面、内部要素)ｘnodeNoのリスト（検索を早めるため）
nodeElsetList = []      #nodeNo☓elementNoのリスト（検索を早めるため）

#  printMsg
#    print help
def printMsg():
    msg = """
<abaqus2fistr.py>
abaqusフォーマットのmeshをFrontISTR用mesh(HECMWフォーマット）に変換する。

<使い方>
abaqus2fistr.py <inpファイル>

例：
abaqus2fistr.py name.inp
abaqus2fistr.py name
"""
    print msg


#
#  getNodeElementData
#  ------------------
#    必要なdataを取得
def getNodeElementData(lines):
    item = ""
    header = ""
    data = []
    n = 0
    while n < len(lines):
        line = lines[n]
        if line[0] == "*":
            if len(data) > 0:
                saveData(item, header, data)

            #nodeの取得
            if line[:len("*NODE")] == "*NODE":
                header = line
                item = "NODE"
                length = 4
                data = []
            #要素の取得
            elif line[:len("*ELEMENT")] == "*ELEMENT":
                typeName = getValName(line, "TYPE")
                #四面体1次
                if typeName == "C3D4":
                    header = setValName(line, "TYPE=341")
                    item = "ELEMENT"
                    length = 5
                    data = []
                #四面体2次
                elif typeName == "C3D10":
                    header = setValName(line, "TYPE=342")
                    item = "ELEMENT"
                    length = 11
                    data = []
                #六面体1次
                elif typeName == "C3D8":
                    header = setValName(line, "TYPE=361")
                    item = "ELEMENT"
                    length = 9
                    data = []
                #六面体2次
                elif typeName == "C3D20":
                    header = setValName(line, "TYPE=362")
                    item = "ELEMENT"
                    length = 21
                    data = []
                #五面体1次
                elif typeName == "C3D6":
                    header = setValName(line, "TYPE=351")
                    item = "ELEMENT"
                    length = 7
                    data = []
                #五面体2次
                elif typeName == "C3D15":
                    header = setValName(line, "TYPE=352")
                    item = "ELEMENT"
                    length = 16
                    data = []
                #表面要素（四面体1次）
                elif typeName == "STRI35":
                    header = setValName(line, "TYPE=S341")
                    item = "SURFACE"
                    length = 4
                    data = []
                #表面要素（四面体2次）
                elif typeName == "S6":
                    header = setValName(line, "TYPE=S342")
                    item = "SURFACE"
                    length = 7
                    data = []
                #表面要素（六面体1次）
                elif typeName == "S4R5":
                    header = setValName(line, "TYPE=S361")
                    item = "SURFACE"
                    length = 5
                    data = []
                #表面要素（六面体2次）
                elif typeName == "S8":
                    header = setValName(line, "TYPE=S362")
                    item = "SURFACE"
                    length = 9
                    data = []
                #以外の要素
                else:
                    item = ""
            #要素groupの取得
            elif line[:len("*ELSET")] == "*ELSET":
                header = line
                item = "ELSET"
                length = 1
                data = []
            #nodeGroupの取得
            elif line[:len("*NSET")] == "*NSET":
                header = line
                item = "NSET"
                length = 1
                data = []
            else:
                item = ""
            n += 1
        else:
            if item != "":
                [data, n] = getDataList(lines, n, item, length)
            else:
                n += 1
    if len(data) > 0:
        saveData(item, header, data)

#  getDataList
#    data部を取得
def getDataList(lines, n, item, length):
    data = []
    if item == "NODE":
        while n < len(lines) and lines[n][0] != "*":
            line = lines[n]
            words = line.split(",")
            if len(words) >= 4:
                a = [int(words[0]), float(words[1]), float(words[2]), float(words[3])]
            else:
                a = ["", "", "", ""]
            data.append(a)
            n += 1
    elif item == "ELEMENT" or item == "SURFACE":
        while n < len(lines) and lines[n][0] != "*":
            line = lines[n]
            words = line.split(ls)[0].split(",")
            for word in words:
                if len(word) > 0:
                    data.append(int(word))
            n += 1
        ans = []
        line = []
        p = 0
        for a in data:
            if p < length:
                line.append(a)
            else:
                ans.append(line)
                p = 0
                line = []
                line.append(a)
            p += 1
        ans.append(line)
        data = ans
    elif item == "ELSET" or item == "NSET":
        while n < len(lines) and lines[n][0] != "*":
            line = lines[n]
            words = line.split(ls)[0].split(",")
            for word in words:
                if len(word) > 0:
                    data.append(int(word))
            n += 1
    return [data, n]

#  saveData
#    読み込んだdataを取得する
def saveData(item, header, data):
    global nodeList, elementList, surfaceList, elsetList, ndsetList
    if item == "NODE":
        block = []
        num = str(len(data))
        header = "!NODE, NUM=" + num + ls
        block.append(header)
        block.append(data)
        nodeList.append(block)
        print "found NODE... num " + num
    elif item == "ELEMENT":
        block = []
        num = str(len(data))
        header = "!" + header[1:]
        header = deleteSp(header).split()[0]
        words = header.split(",")
        header = ", ".join(words) + ", NUM=" + num + ls
        block.append(header)
        block.append(data)
        elementList.append(block)
        name = getValName(header, "TYPE")
        print "found ELEMENT... TYPE " + name + " num " + num
    elif item == "SURFACE":
        block = []
        num = str(len(data))
        header = "!" + header[1:]
        header = deleteSp(header).split()[0]
        words = header.split(",")
        header = ", ".join(words) + ", NUM=" + num + ls
        block.append(header)
        block.append(data)
        surfaceList.append(block)
        name = getValName(header, "TYPE")
        print "found surfaceELEMENT... TYPE " + name + " num " + num
    elif item == "ELSET":
        block = []
        num = str(len(data))
        header = "!" + header[1:]
        header = deleteSp(header).split()[0]
        words = header.split(",")
        header = ", ".join(words) + ", NUM=" + num + ls
        block.append(header)
        block.append(data)
        elsetList.append(block)
        name = getValName(header, "ELSET")
        print "found ELSET... " + name + " num " + num
    elif item == "NSET":
        block = []
        num = str(len(data))
        header = "!" + header[1:]
        header = deleteSp(header).split()[0]
        words = header.split(",")
        header = ", ".join(words) + ", NUM=" + num + ls
        block.append(header)
        block.append(data)
        ndsetList.append(block)
        name = getValName(header, "NSET")
        print "found NSET... " + name + " num " + num


#
#  checkElementListOrder
#  ---------------------
#    要素の並び順をチェック
def checkElementListOrder():
    global elementList, nElementType
    #elementの種類の個数を取得
    nElementType = len(elementList)
    #メッシュの並び順をチェック
    i = 0
    while i < len(elementList):
        element = elementList[i]
        header = element[0]
        typeName = getValName(header, "TYPE")
        #四面体2次
        if typeName == "342":
            data = changeNodeOrder342(element[1])
            elementList[i][1] = data
        #六面体1次
        elif typeName == "361":
            data = changeNodeOrder361(element[1])
            elementList[i][1] = data
        #六面体2次
        elif typeName == "362":
            data = changeNodeOrder362(element[1])
            elementList[i][1] = data
        #5面体1次
        elif typeName == "351":
            data = changeNodeOrder351(element[1])
            elementList[i][1] = data
        #5面体2次
        elif typeName == "352":
            data = changeNodeOrder352(element[1])
            elementList[i][1] = data
        i += 1
    #headerの修正
    remakeElementHeaderData()


#  changeNodeOrder342
#    四面体2次メッシュの節点順を変更
def changeNodeOrder342(data):
    i = 0
    while i < len(data):
        nums = data[i]
        newOder = [nums[0], nums[1], nums[2], nums[3], nums[4], nums[6], nums[7],
                nums[5], nums[8], nums[9], nums[10]]
        data[i] = newOder
        i += 1
    return data

#  changeNodeOrder361
#    六面体1次メッシュの節点順を変更
def changeNodeOrder361(data):
    i = 0
    while i < len(data):
        nums = data[i]
        newOder = [nums[0], nums[1], nums[5], nums[6], nums[2], nums[4], nums[8],
                nums[7], nums[3]]
        data[i] = newOder
        i += 1
    return data

#  changeNodeOrder362
#    六面体2次メッシュの節点順を変更
def changeNodeOrder362(data):
    i = 0
    while i < len(data):
        nums = data[i]
        newOder = [nums[0], nums[1], nums[5], nums[6], nums[2], nums[4], nums[8],
                nums[7], nums[3], nums[17], nums[13], nums[18], nums[9], nums[20],
                nums[15], nums[19], nums[11], nums[12], nums[16], nums[14], nums[10]]
        data[i] = newOder
        i += 1
    return data

#  changeNodeOrder351
#    五面体1次メッシュの節点順を変更
def changeNodeOrder351(data):
    i = 0
    while i < len(data):
        nums = data[i]
        newOrder = [nums[0], nums[1], nums[3], nums[2], nums[4], nums[6], nums[5]]
        data[i] = newOrder
        i += 1
    return data

#  changeNodeOrder352
#    五面体1次メッシュの節点順を変更
def changeNodeOrder352(data):
    i = 0
    while i < len(data):
        nums = data[i]
        newOrder = [nums[0], nums[1], nums[3], nums[2], nums[4], nums[6], nums[5],
                nums[8], nums[7], nums[9], nums[11], nums[10], nums[12], nums[13],
                nums[15], nums[14]]
        data[i] = newOrder
        i += 1
    return data


#
#  remakeElementHeaderData
#    data整形
def remakeElementHeaderData():
    global elementList
    i = 0
    while i < len(elementList):
        header = elementList[i][0]
        typeName = getValName(header, "TYPE")
        num = getValName(header, "NUM")
        name = abaName.split(".")[0]
        header = "!ELEMENT, TYPE=" + typeName + ", PARTNAME=" + name + ", EGRP=" + name + ", NUM=" + num + ls
        elementList[i][0] = header
        i += 1

#
#  checkSurfaceElementGroup
#  ------------------------
#    surfaceのgroup分けをチェック
def checkSurfaceElementGroup():
    print "checking data..."
    global surGroupList, elsetList, surfaceList

    #surfaceのlistを作成
    createSurfaceNodesList()

    #elsetをEGROUPとSGROUPに分ける
    surGroupList = []       #表面要素（surface）
    newElset = []           #内部要素（element）
    for elset in elsetList:
        surfaceNo = elset[1][0]
        if elNodeList[surfaceNo][0][0] == "S":
            #表面要素
            surGroupList += [elset]
        else:
            #内部要素
            newElset += [elset]
    elsetList = newElset
    #その他の表面要素を取得
    getOtherSurGroupList()
    #nodeを含むelementのlistを作成
    createNodeElementSet()

    print "creating SGROUP(surface)...",
    newSurElset = addNodesSurElset(surGroupList)        #各要素にnodeNoを追加
    surfaceGroup = getElementFaceNo(newSurElset)        #elementを取得
    surGroupList = remakeHeaderSurGroup(surfaceGroup)   #header部を修正
    print

#
#  getOtherSurGroupList
#    その他の表面要素を取得
def getOtherSurGroupList():
    global surGroupList
    #配列を準備
    dummy = []
    for surface in surfaceList:
        for surNodes in surface[1]:
            dummy.append(surNodes[0])
    maxSurNo = max(dummy)
    surElement = []
    i = 0
    while i < maxSurNo+1:
        surElement.append(-1)
        i += 1
    #elNoを取得
    for surface in surfaceList:
        for surNodes in surface[1]:
            elNo = surNodes[0]
            surElement[elNo] = 0
    #設定済の表面要素Noを取得
    for surGroup in surGroupList:
        for surElements in surGroup[1]:
            elNo = surElements
            surElement[elNo] = 1
    #その他の表面要素を取得
    otherData = []
    i = 0
    for surElNo in surElement:
        if surElNo == 0:
            otherData.append(i)
        i += 1
    #otherSのelsetを作成
    addList = []
    addList += ["!ELSET, ELSET=otherS, NUM=" + str(len(otherData)) + ls]
    addList += [otherData]
    surGroupList = [addList] + surGroupList

#
#  createNodeElementSet
#    nodeNoをindexにしてそのnodeを含むelementのlist作成
def createNodeElementSet():
    global nodeElsetList
    tmp = []
    for nodes in nodeList:
        for nodeLine in nodes[1]:
            tmp.append(nodeLine[0])
    maxNo = max(tmp)
    #配列の作成
    nodeElsetList = []
    i = 0
    while i < maxNo+1:
        nodeElsetList += [[]]
        i += 1
    #elementNoを取得
    for elements in elementList:
        for nodesLine in elements[1]:
            elNo = nodesLine[0]
            nodes = nodesLine[1:]
            for node in nodes:
                nodeElsetList[node] += [elNo]

#
#  createSurfaceNodesList
#    表面要素の要素NoとnodeNoのlist作成
#    [[<"S" or "E">, elType], [node1, node2,...]]
def createSurfaceNodesList():
    global elNodeList
    #最大要素Noを取得
    tmp = []
    for surface in surfaceList:
        for surfaceNode in surface[1]:
            tmp.append(surfaceNode[0])
    for element in elementList:
        for elementNode in element[1]:
            tmp.append(elementNode[0])
    nMaxSurNo = max(tmp)
    #list作成
    elNodeList = []
    i = 0
    while i < nMaxSurNo+1:
        elNodeList.append([])
        i += 1
    #表面要素、要素typeを保存
    for surface in surfaceList:
        header = surface[0]
        elType = getValName(header, "TYPE")
        for surfaceNode in surface[1]:
            elNo = surfaceNode[0]
            elNodeList[elNo] = [["S", elType]] + surfaceNode[1:]
    #内部要素、要素typeを保存
    for element in elementList:
        header = element[0]
        elType = getValName(header, "TYPE")
        for elementNode in element[1]:
            elNo = elementNode[0]
            elNodeList[elNo] = [["E", elType]] + elementNode[1:]

#  separateElset
#    elsetをEGROUPとSGROUPに分ける
def separateElset():
    global surfaceList, surGroupList, elsetList
    surGroupList = []
    newElset = []
    i = 0
    while i < len(elsetList):
        elset = elsetList[i]
        surfaceNo = elset[1][0]
        flag = 0
        for surElement in surfaceList:
            for surElNo in surElement[1]:
                if surfaceNo == surElNo[0]:
                    flag = 1
                    break
            if flag == 1:
                break
        if flag == 1:
            surGroupList += [elset]
        else:
            newElset += [elset]
        i += 1
    elsetList = newElset
    return

#  addNodesSurElset
#    各表面要素にnode番号を追加
def addNodesSurElset(elsets):
    newSurElset = []
    for elset in elsets:
        data = []
        for elNo in elset[1]:
            nodes = elNodeList[elNo][1:]
            newNodes = [elNo] + nodes
            data.append(newNodes)
        data = [elset[0]] + [data]
        newSurElset.append(data)
    return newSurElset

#  findSurfaceNodes
def findSurfaceNodes(elNo):
    ans = []
    flag = 0
    for surface in surfaceList:
        for nodeNo in surface[1]:
            if nodeNo[0] == elNo:
                flag = 1
                ans = nodeNo[1:]
                break
        if flag == 1:
            break
    return ans

#  getElementFaceNo
#    elementとfaceNo取得
def getElementFaceNo(elsets):
    #elementNo取得
    elementNoSet = []
    for elset in elsets:
        header = elset[0]
        name = getValName(header, "ELSET")
        print name,
        data = []
        ellNo = 0
        for line in elset[1]:
            setNodes = line[1:]
            [elementNo, faceNo] = findElementNo(setNodes)
            #data.append([elementNo] + [faceNo])
            data += [[elementNo] + [faceNo]]
        data.sort()
        #elementNoSet.append([header] + [data])
        elementNoSet += [[header] + [data]]
    return elementNoSet


#  findElementNo
#    elementNoを取得し、そのfaceNoを取得して返す
def findElementNo(nodes):
    flag = 0
    i = 0
    #最初のnodeNoを含むelementListを取得
    elements = nodeElsetList[nodes[0]]
    while i < len(elements):
        if i < len(elements):
            elementNo = elements[i]
            line = elNodeList[elementNo]
            if isInNodes(line[1:], nodes) == True:
                elType = elNodeList[elementNo][0][1]
                faceNo = getFaceNoOfElement(line[1:], nodes, elType)
                flag = 1
                break
        i += 1
    return [elementNo, faceNo]

#  isInNodes
def isInNodes(line, nodes):
    flag = 0
    ans = False
    for node in nodes:
        if node in line:
            flag += 1
        else:
            break
    if flag == len(nodes):
        ans = True
    return ans

#  remakeHeaderSurGroup
#    headerを書き換え
def remakeHeaderSurGroup(elsets):
    i = 0
    while i < len(elsets):
        elset = elsets[i]
        header = elset[0]
        groupName = getValName(header, "ELSET")
        num = getValName(header, "NUM")
        partName = abaName.split(".")[0]
        header = "!SGROUP, PARTNAME=" + name + ", SGRP=" + groupName + ", NUM=" + num + ls
        elsets[i][0] = header
        i += 1
    return elsets

#  getFaceNoOfElement
#    elementのfaceNo取得
def getFaceNoOfElement(elNodes, faceNodes, elType):
    faceNo = 0
    #四面体（1次、2次）要素の場合
    if elType == "341" or elType == "342":
        nodes = [1,2,3,4]
        i = 0
        newNodes = []
        while i < len(nodes):
            if elNodes[i] in faceNodes:
                newNodes.append(i+1)
            i += 1
        """
        while i < len(nodes):
            for faceNode in faceNodes:
                if elNodes[i] == faceNode:
                    newNodes.append(nodes[i])
                    break
            i += 1
        """
        newNodes.sort()
        if newNodes == [1,2,3]:
            faceNo = 1
        elif newNodes == [1,2,4]:
            faceNo = 2
        elif newNodes == [2,3,4]:
            faceNo = 3
        elif newNodes == [1,3,4]:
            faceNo = 4
        else:
            faceNo = 0
    #六面体（1次、2次）要素の場合
    elif elType == "361" or elType == "362":
        nodes = [1,2,3,4,5,6,7,8]
        i = 0
        newNodes = []
        while i < len(nodes):
            for faceNode in faceNodes:
                if elNodes[i] == faceNode:
                    newNodes.append(nodes[i])
                    break
            i += 1
        newNodes.sort()
        if newNodes == [1,2,3,4]:
            faceNo = 1
        elif newNodes == [5,6,7,8]:
            faceNo = 2
        elif newNodes == [1,2,5,6]:
            faceNo = 3
        elif newNodes == [2,3,6,7]:
            faceNo = 4
        elif newNodes == [3,4,7,8]:
            faceNo = 5
        elif newNodes == [1,4,5,8]:
            faceNo = 6
        else:
            faceNo = 0
    #五面体（1次、2次）要素の場合
    elif elType == "351" or elType == "352":
        nodes = [1,2,3,4,5,6]
        i = 0
        newNodes = []
        while i < len(nodes):
            for faceNode in faceNodes:
                if elNodes[i] == faceNode:
                    newNodes.append(nodes[i])
                    break
            i += 1
        newNodes.sort()
        if newNodes == [1,2,3]:
            faceNo = 1
        elif newNodes == [4,5,6]:
            faceNo = 2
        elif newNodes == [1,2,4,5]:
            faceNo = 3
        elif newNodes == [2,3,5,6]:
            faceNo = 4
        elif newNodes == [1,3,4,6]:
            faceNo = 5
        else:
            faceNo = 0
    return faceNo

#
#  checkElementGroup
#  -----------------
#    elementのgroup分けをチェック
def checkElementGroup():
    global elsetList, elementList, nElementType
    if len(elsetList) == 0:
        return

    print "creating EGROUP(element)...",
    #elementTypeの数は？
    if nElementType > 1:
        #複数のelementTypeがある場合
        newElGroup = []
        for elset in elsetList:
            newData = createNewElGroup(elset)
            newElGroup.append(newData)
        elsetList = newElGroup
        print
        return

    #elementTypeが1種類の場合
    elSetList = []
    newElementList = []
    flag = 0
    for elset in elsetList:
        flag = 1
        newData = createNewElementData(elset)
        newElementList.append(newData)
    if flag == 1:
        otherData = createOtherElementData(newElementList)
        if len(otherData[1]) > 0:
            newElementList.append(otherData)
        elementList = newElementList
    print

#  createNewElGroup
#    elsetを作成する
def createNewElGroup(elset):
    line = elset[0]
    name = getValName(line, "ELSET")
    partName = abaName.split(".")[0]
    num = getValName(line, "NUM")
    header = "!EGROUP, PARTNAME=" + partName + ", EGRP=" + name + ", NUM=" + num + ls
    elset[0] = header
    return elset

#  createNewElementData
#    elsetのelementDataを作成する
def createNewElementData(elset):
    line = elset[0]
    name = getValName(line, "ELSET")
    line = elementList[0][0]
    header = setValName(line, "EGRP="+name)
    print name,
    ans = []
    newElset = []
    for elNo in elset[1]:
        nodes = elNodeList[elNo][1:]
        newElset += [[elNo] + nodes]
    num = str(len(newElset))
    header = setValName(header, "NUM="+num)
    ans.append(header)
    ans.append(newElset)
    return ans

#  createOtherElementData
#    otherElementを追加する
def createOtherElementData(newElementList):
    nEle = 0
    for element in newElementList:
        nEle += len(element[1])
    if nEle == len(elementList[0][1]):
        ans = ["", []]
        return ans

    #既に設定してあるELSETを取得
    #  配列を準備
    elset = []
    i = 0
    while i < len(elNodeList):
        elset.append(0)
        i += 1
    #  定義済の場合、「1」をセット
    for element in newElementList:
        for elNode in element[1]:
            elNo = elNode[0]
            elset[elNo] = 1

    #otherを取得
    print "other",
    otherData = []
    for elLine in elementList[0][1]:
        flag = 0
        elNo = elLine[0]
        if elset[elNo] == 0:
            otherData.append(elLine)
    header = elementList[0][0]
    header= setValName(header, "EGRP=other")
    num = str(len(otherData))
    header = setValName(header, "NUM="+num)
    ans = [header]
    ans.append(otherData)
    return ans

#
#  checkNodeGroup
#  --------------
#    nodeGroupをチェック
def checkNodeGroup():
    global ndsetList
    print "creating NGROUP(node)...",
    newNdset = []
    for ndSet in ndsetList:
        data = []
        header = ndSet[0]
        name = getValName(header, "NSET")
        print name,
        num = getValName(header, "NUM")
        partName = abaName.split(".")[0]
        header = "!NGROUP, PARTNAME=" + partName + ", NGRP=" + name + ", NUM=" + num + ls
        data.append(header)
        data.append(ndSet[1])
        data[1].sort()
        newNdset.append(data)
    ndsetList = newNdset
    print

#
#  witeData
#  --------
#    dataの書き込み
def writeData(fileName):
    print "writing...",
    cont = makeHeader()
    cont += makeNode()
    cont += makeElement()
    cont += makeElementGroup()
    cont += makeSurfaceGroup()
    cont += makeNodeGroup()
    cont += "!END"
    f=open(fileName, "w"); f.write(cont); f.close()
    print "done"

#  makeHeader
#    header
def makeHeader():
    cont = "!HEADER" + ls
    cont += "  genarated by abaqus2fistr.py" + ls
    return cont

#  makeNode
#    node
def makeNode():
    cont = ""
    for node in nodeList:
        cont += node[0]
        for nums in node[1]:
            strs = []
            for num in nums:
                strs.append(str(num))
            cont += ", ".join(strs) + ls
    return cont        

#  makeElement
#    element
def makeElement():
    cont = ""
    for element in elementList:
        header = element[0]
        name = getValName(header, "EGRP")
        cont += header
        for nums in element[1]:
            strs = []
            for num in nums:
                strs.append(str(num))
            cont += ", ".join(strs) + ls
        section = "!SECTION, TYPE=SOLID, EGRP=" + name + ls
        cont += section
    return cont

#  makeElementGroup
#    elementGroup
def makeElementGroup():
    cont = ""
    if nElementType == 1:
        return cont

    for elGroup in elsetList:
        cont += elGroup[0]
        cont += makeStringFromData(elGroup[1], 8)
    return cont

#  makeSurfaceGroup
#    表面要素group
def makeSurfaceGroup():
    cont = ""
    for surGroup in surGroupList:
        cont += surGroup[0]
        for surSet in surGroup[1]:
            strs = []
            for num in surSet:
                strs.append(str(num))
            cont += ", ".join(strs) + ls
    return cont

#  makeNodeGroup
#    nodeGroup
def makeNodeGroup():
    cont = ""
    if len(ndsetList) == 0:
        return cont

    for ndset in ndsetList:
        cont += ndset[0]
        cont += makeStringFromData(ndset[1], 8)
    return cont

#  makeStringFromData
#    dataSetをstrに変換
def makeStringFromData(data, w):
    cont = ""
    p = 0
    while p+w < len(data):
        nums = data[p:p+w]
        strs = []
        for num in nums:
            strs.append(str(num))
        cont += ", ".join(strs) + "," + ls
        p += w
    nums = data[p:]
    strs = []
    for num in nums:
        strs.append(str(num))
    cont += ", ".join(strs) + ls
    return cont

#  getValName
#    valName(TYPEなど）を取得
def getValName(line, valName):
    words = line.split(",")
    name = ""
    for word in words:
        # 「TYPE=」などを検索
        if word.find(valName+"=") >= 0:
            name = word.split("=")[1]
            break
    newName = ""
    for chara in name:
        if chara > " ":
            newName += chara
    return newName

#  deleteSp
#    空白を削除
def deleteSp(line):
    newLine = ""
    for chara in line:
        if chara != " ":
            newLine += chara
    return newLine


#  setValName
#    変数の値をセット
def setValName(line, val):
    valNames = val.split("=")
    words = (line.split(ls)[0]).split(",")
    i = 0
    flag = 0
    for word in words:
        name = (word.split()[0]).split("=")[0]
        if name == valNames[0]:
            words[i] = " " + val
            flag = 1
            break
        i += 1
    if flag == 0:
        words.append(" "+val)
    newLine = ",".join(words) + ls
    return newLine

#
#  abaqus2fistr
#  ------------
#    convert abaqus to fistr
def abaqus2fistr(fileDir, abaName):
    #fileName = fileDir + "/" + abaName + ".inp"
    fileName = fileDir + os.sep + abaName + ".inp"
    f=open(fileName); lines=f.readlines(); f.close()
    #データ取得
    getNodeElementData(lines)
    #nodeの並びをチェック
    checkElementListOrder()
    #表面groupの取得
    checkSurfaceElementGroup()
    #要素groupの取得（単一要素の場合は、要素定義をgroup分けする）
    checkElementGroup()
    #nodeGroupの取得
    checkNodeGroup()
    #書き出し
    #newName = fileDir + "/" + abaName + ".msh"
    newName = fileDir + os.sep + abaName + ".msh"
    writeData(newName)


if __name__=='__main__':
    abaNameDir = sys.argv[1]
    if abaNameDir[:len("-h")] == "-h" or abaNameDir[:len("--help")] == "--help":
        printMsg()
    else:
        #fileDir = "/".join(abaNameDir.split("/")[:-1])
        fileDir = os.sep.join(abaNameDir.split(os.sep)[:-1])
        if fileDir == "":
            fileDir = os.getcwd()
        abaName = abaNameDir.split(os.sep)[-1]
        if abaName.split(".")[-1] == "inp":
            name = abaName[:-4]
        else:
            name = abaName
        abaqus2fistr(fileDir, name)

